@@ -1,3 +1,5 @@
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 #run unbound
 service unbound start && service unbound status;
 # dig @localhost -p 53535
@@ -7,5 +9,5 @@ service bind9 start && service bind9 status;
 # dig @localhost -p 53533
 
 #run kresd
-LD_LIBRARY_PATH=$PREFIX/lib $PREFIX/sbin/kresd -f 1 -q -c $(pwd)/ci/respdiff/kresd.config &>kresd.log &
+$PREFIX/sbin/kresd -n -q -c $(pwd)/ci/respdiff/kresd.config &>kresd.log &
 # dig @localhost -p 5353
