@@ -3,14 +3,14 @@
  *
  *	(c) 1997--2015 Martin Mares <mj@ucw.cz>
  *	(c) 2007 Pavel Charvat <pchar@ucw.cz>
- *
- *	This software may be freely distributed and used according to the terms
- *	of the GNU Lesser General Public License.
+ *	SPDX-License-Identifier: LGPL-2.1-or-later
+ *	Source: https://www.ucw.cz/libucw/
  */
 
 #ifndef _UCW_POOLS_H
 #define _UCW_POOLS_H
 
+#include "lib/defines.h"
 #include <ucw/alloc.h>
 #include <ucw/config.h>
 #include <ucw/lib.h>
@@ -100,6 +100,7 @@ struct mempool_stats {			/** Mempool statistics. See @mp_stats(). **/
  *
  * Memory pools can be treated as <<trans:respools,resources>>, see <<trans:res_mempool()>>.
  **/
+KR_EXPORT
 void mp_init(struct mempool *pool, size_t chunk_size);
 
 /**
@@ -110,6 +111,7 @@ void mp_init(struct mempool *pool, size_t chunk_size);
  *
  * Memory pools can be treated as <<trans:respools,resources>>, see <<trans:res_mempool()>>.
  **/
+KR_EXPORT
 struct mempool *mp_new(size_t chunk_size);
 
 /**
@@ -117,6 +119,7 @@ struct mempool *mp_new(size_t chunk_size);
  * Frees all the memory allocated by this mempool and,
  * if created by @mp_new(), the @pool itself.
  **/
+KR_EXPORT
 void mp_delete(struct mempool *pool);
 
 /**
@@ -125,6 +128,7 @@ void mp_delete(struct mempool *pool);
  * further allocation requests. Leaves the @pool alive,
  * even if it was created with @mp_new().
  **/
+KR_EXPORT
 void mp_flush(struct mempool *pool);
 
 /**
@@ -166,6 +170,7 @@ void *mp_alloc_internal(struct mempool *pool, size_t size) LIKE_MALLOC;
  * `CPU_STRUCT_ALIGN` bytes and this condition remains true also
  * after future reallocations.
  **/
+KR_EXPORT
 void *mp_alloc(struct mempool *pool, size_t size);
 
 /**
@@ -531,6 +536,7 @@ char *mp_str_from_mem(struct mempool *p, const void *mem, size_t len) LIKE_MALLO
 /**
  * printf() into a in-memory string, allocated on the memory pool.
  **/
+KR_EXPORT
 char *mp_printf(struct mempool *mp, const char *fmt, ...) FORMAT_CHECK(printf,2,3) LIKE_MALLOC;
 /**
  * Like @mp_printf(), but uses `va_list` for parameters.
@@ -549,6 +555,7 @@ char *mp_vprintf(struct mempool *mp, const char *fmt, va_list args) LIKE_MALLOC;
  * not called on an opened growing buffer. The old name will be preserved for backward
  * compatibility for the time being.
  **/
+KR_EXPORT
 char *mp_printf_append(struct mempool *mp, char *ptr, const char *fmt, ...) FORMAT_CHECK(printf,3,4);
 #define mp_append_printf mp_printf_append
 /**
