@@ -5,6 +5,7 @@
 #pragma once
 
 #include "daemon/engine.h"
+#include "daemon/worker.h" /* the_worker is often useful */
 
 #include <lua.h>
 #include <lauxlib.h>
@@ -15,7 +16,7 @@
 #endif
 
 
-/** Useful to stringify #defines into error strings. */
+/** Useful to stringify macros into error strings. */
 #define STR(s) STRINGIFY_TOKEN(s)
 #define STRINGIFY_TOKEN(s) #s
 
