@@ -1,4 +1,4 @@
-/*  Copyright (C) 2015-2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015-2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
  *  SPDX-License-Identifier: GPL-3.0-or-later
  */
 
@@ -7,7 +7,6 @@
 #include "contrib/base64.h"
 #include "daemon/network.h"
 #include "daemon/tls.h"
-#include "daemon/worker.h"
 
 #include <stdlib.h>
 
@@ -23,8 +22,12 @@ static int net_list_add(const char *key, void *val, void *ext)
 
 		if (ep->flags.kind) {
 			lua_pushstring(L, ep->flags.kind);
+		} else if (ep->flags.http && ep->flags.tls) {
+			lua_pushliteral(L, "doh2");
 		} else if (ep->flags.tls) {
 			lua_pushliteral(L, "tls");
+		} else if (ep->flags.xdp) {
+			lua_pushliteral(L, "xdp");
 		} else {
 			lua_pushliteral(L, "dns");
 		}
@@ -39,6 +42,9 @@ static int net_list_add(const char *key, void *val, void *ext)
 		case AF_INET6:
 			lua_pushliteral(L, "inet6");
 			break;
+		case AF_XDP:
+			lua_pushliteral(L, "inet4+inet6"); // both UDP ports at once
+			break;
 		case AF_UNIX:
 			lua_pushliteral(L, "unix");
 			break;
@@ -49,17 +55,21 @@ static int net_list_add(const char *key, void *val, void *ext)
 		lua_setfield(L, -2, "family");
 
 		lua_pushstring(L, key);
-		if (ep->family != AF_UNIX) {
+		if (ep->family == AF_INET || ep->family == AF_INET6) {
 			lua_setfield(L, -2, "ip");
-		} else {
+			lua_pushboolean(L, ep->flags.freebind);
+			lua_setfield(L, -2, "freebind");
+		} else if (ep->family == AF_UNIX) {
 			lua_setfield(L, -2, "path");
+		} else if (ep->family == AF_XDP) {
+			lua_setfield(L, -2, "interface");
+			lua_pushinteger(L, ep->nic_queue);
+			lua_setfield(L, -2, "nic_queue");
 		}
 
 		if (ep->family != AF_UNIX) {
 			lua_pushinteger(L, ep->port);
 			lua_setfield(L, -2, "port");
-			lua_pushboolean(L, ep->flags.freebind);
-			lua_setfield(L, -2, "freebind");
 		}
 
 		if (ep->family == AF_UNIX) {
@@ -86,19 +96,20 @@ static int net_list_add(const char *key, void *val, void *ext)
 /** List active endpoints. */
 static int net_list(lua_State *L)
 {
-	struct engine *engine = engine_luaget(L);
 	lua_newtable(L);
 	lua_pushinteger(L, 1);
-	map_walk(&engine->net.endpoints, net_list_add, L);
+	map_walk(&the_worker->engine->net.endpoints, net_list_add, L);
 	lua_pop(L, 1);
 	return 1;
 }
 
 /** Listen on an address list represented by the top of lua stack.
- * \note kind ownership is not transferred
+ * \note flags.kind ownership is not transferred, and flags.sock_type doesn't make sense
  * \return success */
-static bool net_listen_addrs(lua_State *L, int port, bool tls, const char *kind, bool freebind)
+static bool net_listen_addrs(lua_State *L, int port, endpoint_flags_t flags, int16_t nic_queue)
 {
+	assert(flags.xdp || nic_queue == -1);
+
 	/* Case: table with 'addr' field; only follow that field directly. */
 	lua_getfield(L, -1, "addr");
 	if (!lua_isnil(L, -1)) {
@@ -110,33 +121,56 @@ static bool net_listen_addrs(lua_State *L, int port, bool tls, const char *kind,
 	/* Case: string, representing a single address. */
 	const char *str = lua_tostring(L, -1);
 	if (str != NULL) {
-		struct engine *engine = engine_luaget(L);
+		struct network *net = &the_worker->engine->net;
+		const bool is_unix = str[0] == '/';
 		int ret = 0;
-		endpoint_flags_t flags = { .tls = tls, .freebind = freebind };
-		if (!kind && !flags.tls) { /* normal UDP */
+		if (!flags.kind && !flags.tls) { /* normal UDP or XDP */
 			flags.sock_type = SOCK_DGRAM;
-			ret = network_listen(&engine->net, str, port, flags);
+			ret = network_listen(net, str, port, nic_queue, flags);
 		}
-		if (!kind && ret == 0) { /* common for normal TCP and TLS */
+		if (!flags.kind && !flags.xdp && ret == 0) { /* common for TCP, DoT and DoH (v2) */
 			flags.sock_type = SOCK_STREAM;
-			ret = network_listen(&engine->net, str, port, flags);
+			ret = network_listen(net, str, port, nic_queue, flags);
 		}
-		if (kind) {
-			flags.kind = strdup(kind);
+		if (flags.kind) {
+			flags.kind = strdup(flags.kind);
 			flags.sock_type = SOCK_STREAM; /* TODO: allow to override this? */
-			ret = network_listen(&engine->net, str, port, flags);
+			ret = network_listen(net, str, (is_unix ? 0 : port), nic_queue, flags);
 		}
-		if (ret != 0) {
-			if (str[0] == '/') {
-				kr_log_error("[system] bind to '%s' (UNIX): %s\n",
-						str, kr_strerror(ret));
-			} else {
-				const char *stype = flags.sock_type == SOCK_DGRAM ? "UDP" : "TCP";
-				kr_log_error("[system] bind to '%s@%d' (%s): %s\n",
-						str, port, stype, kr_strerror(ret));
+		if (ret == 0) return true; /* success */
+
+		if (is_unix) {
+			kr_log_error("[system] bind to '%s' (UNIX): %s\n",
+					str, kr_strerror(ret));
+		} else if (flags.xdp) {
+			const char *err_str = knot_strerror(ret);
+			if (ret == KNOT_ELIMIT) {
+				if ((strcmp(str, "::") == 0 || strcmp(str, "0.0.0.0") == 0)) {
+					err_str = "wildcard addresses not supported with XDP";
+				} else {
+					err_str = "address matched multiple network interfaces";
+				}
+			} else if (ret == kr_error(ENODEV)) {
+				err_str = "invalid address or interface name";
 			}
+			/* Notable OK strerror: KNOT_EPERM Operation not permitted */
+
+			if (nic_queue == -1) {
+				kr_log_error("[system] failed to initialize XDP for '%s@%d'"
+						" (nic_queue = <auto>): %s\n",
+						str, port, err_str);
+			} else {
+				kr_log_error("[system] failed to initialize XDP for '%s@%d'"
+						" (nic_queue = %d): %s\n",
+						str, port, nic_queue, err_str);
+			}
+
+		} else {
+			const char *stype = flags.sock_type == SOCK_DGRAM ? "UDP" : "TCP";
+			kr_log_error("[system] bind to '%s@%d' (%s): %s\n",
+					str, port, stype, kr_strerror(ret));
 		}
-		return ret == 0;
+		return false; /* failure */
 	}
 
 	/* Last case: table where all entries are added recursively. */
@@ -144,7 +178,7 @@ static bool net_listen_addrs(lua_State *L, int port, bool tls, const char *kind,
 		lua_error_p(L, "bad type for address");
 	lua_pushnil(L);
 	while (lua_next(L, -2)) {
-		if (!net_listen_addrs(L, port, tls, kind, freebind))
+		if (!net_listen_addrs(L, port, flags, nic_queue))
 			return false;
 		lua_pop(L, 1);
 	}
@@ -183,40 +217,64 @@ static int net_listen(lua_State *L)
 		}
 	}
 
-	bool tls = (port == KR_DNS_TLS_PORT);
-	bool freebind = false;
-	const char *kind = NULL;
+	endpoint_flags_t flags = { 0 };
+	if (port == KR_DNS_TLS_PORT) {
+		flags.tls = true;
+	} else if (port == KR_DNS_DOH_PORT) {
+		flags.http = flags.tls = true;
+	}
+
+	int16_t nic_queue = -1;
 	if (n > 2 && !lua_isnil(L, 3)) {
 		if (!lua_istable(L, 3))
 			lua_error_p(L, "wrong type of third parameter (table expected)");
-		tls = table_get_flag(L, 3, "tls", tls);
-		freebind = table_get_flag(L, 3, "freebind", tls);
+		flags.tls = table_get_flag(L, 3, "tls", flags.tls);
+		flags.freebind = table_get_flag(L, 3, "freebind", false);
 
 		lua_getfield(L, 3, "kind");
 		const char *k = lua_tostring(L, -1);
 		if (k && strcasecmp(k, "dns") == 0) {
-			tls = false;
-		} else
-		if (k && strcasecmp(k, "tls") == 0) {
-			tls = true;
-		} else
-		if (k) {
-			kind = k;
+			flags.tls = flags.http = false;
+		} else if (k && strcasecmp(k, "xdp") == 0) {
+			flags.tls = flags.http = false;
+			flags.xdp = true;
+		} else if (k && strcasecmp(k, "tls") == 0) {
+			flags.tls = true;
+			flags.http = false;
+		} else if (k && strcasecmp(k, "doh2") == 0) {
+			flags.tls = flags.http = true;
+		} else if (k) {
+			flags.kind = k;
+			if (strcasecmp(k, "doh") == 0) {
+				kr_log_deprecate(
+					"kind=\"doh\" is an obsolete DoH implementation, use kind=\"doh2\" instead\n");
+			}
+		}
+
+		lua_getfield(L, 3, "nic_queue");
+		if (lua_isnumber(L, -1)) {
+			if (flags.xdp) {
+				nic_queue = lua_tointeger(L, -1);
+			} else {
+				lua_error_p(L, "nic_queue only supported with kind = 'xdp'");
+			}
+		} else if (!lua_isnil(L, -1)) {
+			lua_error_p(L, "wrong value of nic_queue (integer expected)");
 		}
 	}
 
 	/* Memory management of `kind` string is difficult due to longjmp etc.
 	 * Pop will unreference the lua value, so we store it on C stack instead (!) */
-	const int kind_alen = kind ? strlen(kind) + 1 : 1 /* 0 length isn't C standard */;
+	const int kind_alen = flags.kind ? strlen(flags.kind) + 1 : 1 /* 0 length isn't C standard */;
 	char kind_buf[kind_alen];
-	if (kind) {
-		memcpy(kind_buf, kind, kind_alen);
-		kind = kind_buf;
+	if (flags.kind) {
+		memcpy(kind_buf, flags.kind, kind_alen);
+		flags.kind = kind_buf;
 	}
 
 	/* Now focus on the first argument. */
 	lua_settop(L, 1);
-	if (!net_listen_addrs(L, port, tls, kind, freebind))
+	if (!net_listen_addrs(L, port, flags, nic_queue))
 		lua_error_p(L, "net.listen() failed to bind");
 	lua_pushboolean(L, true);
 	return 1;
@@ -240,8 +298,7 @@ static int net_close(lua_State *L)
 	if (!ok)
 		lua_error_p(L, "expected 'close(string addr, [number port])'");
 
-	struct network *net = &engine_luaget(L)->net;
-	int ret = network_close(net, addr, port);
+	int ret = network_close(&the_worker->engine->net, addr, port);
 	lua_pushboolean(L, ret == 0);
 	return 1;
 }
@@ -301,16 +358,30 @@ static int net_interfaces(lua_State *L)
 /** Set UDP maximum payload size. */
 static int net_bufsize(lua_State *L)
 {
-	struct engine *engine = engine_luaget(L);
-	knot_rrset_t *opt_rr = engine->resolver.opt_rr;
-	if (!lua_isnumber(L, 1)) {
-		lua_pushinteger(L, knot_edns_get_payload(opt_rr));
-		return 1;
+	struct kr_context *ctx = &the_worker->engine->resolver;
+	const int argc = lua_gettop(L);
+	if (argc == 0) {
+		lua_pushinteger(L, knot_edns_get_payload(ctx->downstream_opt_rr));
+		lua_pushinteger(L, knot_edns_get_payload(ctx->upstream_opt_rr));
+		return 2;
+	}
+
+	if (argc == 1) {
+		int bufsize = lua_tointeger(L, 1);
+		if (bufsize < 512 || bufsize > UINT16_MAX)
+			lua_error_p(L, "bufsize must be within <512, " STR(UINT16_MAX) ">");
+		knot_edns_set_payload(ctx->downstream_opt_rr, (uint16_t)bufsize);
+		knot_edns_set_payload(ctx->upstream_opt_rr, (uint16_t)bufsize);
+	} else if (argc == 2) {
+		int bufsize_downstream = lua_tointeger(L, 1);
+		int bufsize_upstream = lua_tointeger(L, 2);
+		if (bufsize_downstream < 512 || bufsize_upstream < 512
+		    || bufsize_downstream > UINT16_MAX || bufsize_upstream > UINT16_MAX) {
+			lua_error_p(L, "bufsize must be within <512, " STR(UINT16_MAX) ">");
+		}
+		knot_edns_set_payload(ctx->downstream_opt_rr, (uint16_t)bufsize_downstream);
+		knot_edns_set_payload(ctx->upstream_opt_rr, (uint16_t)bufsize_upstream);
 	}
-	int bufsize = lua_tointeger(L, 1);
-	if (bufsize < 512 || bufsize > UINT16_MAX)
-		lua_error_p(L, "bufsize must be within <512, " STR(UINT16_MAX) ">");
-	knot_edns_set_payload(opt_rr, (uint16_t) bufsize);
 	return 0;
 }
 
@@ -335,11 +406,7 @@ static int net_pipeline(lua_State *L)
 
 static int net_tls(lua_State *L)
 {
-	struct engine *engine = engine_luaget(L);
-	if (!engine) {
-		return 0;
-	}
-	struct network *net = &engine->net;
+	struct network *net = &the_worker->engine->net;
 	if (!net) {
 		return 0;
 	}
@@ -474,7 +541,7 @@ static int net_tls_client(lua_State *L)
 	/* TODO idea: allow starting the lua table with *multiple* IP targets,
 	 * meaning the authentication config should be applied to each.
 	 */
-	struct network *net = &engine_luaget(L)->net;
+	struct network *net = &the_worker->engine->net;
 	if (lua_gettop(L) == 0)
 		return tls_params2lua(L, net->tls_client_params);
 	/* Various basic sanity-checking. */
@@ -688,7 +755,7 @@ int net_tls_client_clear(lua_State *L)
 	if (!addr)
 		lua_error_p(L, "invalid IP address");
 	/* Do the actual removal. */
-	struct network *net = &engine_luaget(L)->net;
+	struct network *net = &the_worker->engine->net;
 	int r = tls_client_param_remove(net->tls_client_params, addr);
 	free_const(addr);
 	lua_error_maybe(L, r);
@@ -698,18 +765,18 @@ int net_tls_client_clear(lua_State *L)
 
 static int net_tls_padding(lua_State *L)
 {
-	struct engine *engine = engine_luaget(L);
+	struct kr_context *ctx = &the_worker->engine->resolver;
 
 	/* Only return current padding. */
 	if (lua_gettop(L) == 0) {
-		if (engine->resolver.tls_padding < 0) {
+		if (ctx->tls_padding < 0) {
 			lua_pushboolean(L, true);
 			return 1;
-		} else if (engine->resolver.tls_padding == 0) {
+		} else if (ctx->tls_padding == 0) {
 			lua_pushboolean(L, false);
 			return 1;
 		}
-		lua_pushinteger(L, engine->resolver.tls_padding);
+		lua_pushinteger(L, ctx->tls_padding);
 		return 1;
 	}
 
@@ -720,15 +787,15 @@ static int net_tls_padding(lua_State *L)
 	if (lua_isboolean(L, 1)) {
 		bool x = lua_toboolean(L, 1);
 		if (x) {
-			engine->resolver.tls_padding = -1;
+			ctx->tls_padding = -1;
 		} else {
-			engine->resolver.tls_padding = 0;
+			ctx->tls_padding = 0;
 		}
 	} else if (lua_isnumber(L, 1)) {
 		int padding = lua_tointeger(L, 1);
 		if ((padding < 0) || (padding > MAX_TLS_PADDING))
 			lua_error_p(L, "%s", errstr);
-		engine->resolver.tls_padding = padding;
+		ctx->tls_padding = padding;
 	} else {
 		lua_error_p(L, "%s", errstr);
 	}
@@ -741,7 +808,7 @@ static int net_tls_padding(lua_State *L)
 
 static int net_tls_sticket_secret_string(lua_State *L)
 {
-	struct network *net = &engine_luaget(L)->net;
+	struct network *net = &the_worker->engine->net;
 
 	size_t secret_len;
 	const char *secret;
@@ -807,7 +874,7 @@ static int net_tls_sticket_secret_file(lua_State *L)
 	}
 	fclose(fp);
 
-	struct network *net = &engine_luaget(L)->net;
+	struct network *net = &the_worker->engine->net;
 
 	tls_session_ticket_ctx_destroy(net->tls_session_ticket_ctx);
 	net->tls_session_ticket_ctx =
@@ -896,17 +963,13 @@ static int net_update_timeout(lua_State *L, uint64_t *timeout, const char *name)
 
 static int net_tcp_in_idle(lua_State *L)
 {
-	struct engine *engine = engine_luaget(L);
-	struct network *net = &engine->net;
-
+	struct network *net = &the_worker->engine->net;
 	return net_update_timeout(L, &net->tcp.in_idle_timeout, "net.tcp_in_idle");
 }
 
 static int net_tls_handshake_timeout(lua_State *L)
 {
-	struct engine *engine = engine_luaget(L);
-	struct network *net = &engine->net;
-
+	struct network *net = &the_worker->engine->net;
 	return net_update_timeout(L, &net->tcp.tls_handshake_timeout, "net.tls_handshake_timeout");
 }
 
@@ -919,8 +982,6 @@ static int net_bpf_set(lua_State *L)
 
 #if __linux__
 
-	struct engine *engine = engine_luaget(L);
-	struct network *net = &engine->net;
 	int progfd = lua_tointeger(L, 1);
 	if (progfd == 0) {
 		/* conversion error despite that fact
@@ -930,7 +991,7 @@ static int net_bpf_set(lua_State *L)
 	}
 	lua_pop(L, 1);
 
-	if (network_set_bpf(net, progfd) == 0) {
+	if (network_set_bpf(&the_worker->engine->net, progfd) == 0) {
 		lua_error_p(L, "failed to attach BPF program to some networks: %s",
 				kr_strerror(errno));
 	}
@@ -949,9 +1010,7 @@ static int net_bpf_clear(lua_State *L)
 
 #if __linux__
 
-	struct engine *engine = engine_luaget(L);
-	struct network *net = &engine->net;
-	network_clear_bpf(net);
+	network_clear_bpf(&the_worker->engine->net);
 
 	lua_pushboolean(L, 1);
 	return 1;
@@ -970,7 +1029,7 @@ static int net_register_endpoint_kind(lua_State *L)
 	}
 	size_t kind_len;
 	const char *kind = lua_tolstring(L, 1, &kind_len);
-	struct network *net = &engine_luaget(L)->net;
+	struct network *net = &the_worker->engine->net;
 
 	/* Unregistering */
 	if (param_count == 1) {
