@@ -735,16 +735,10 @@ void io_tty_alloc(uv_handle_t *handle, size_t suggested, uv_buf_t *buf)
 }
 
 struct io_stream_data *io_tty_alloc_data() {
-	knot_mm_t _pool = {
-		.ctx = mp_new(4096),
-		.alloc = (knot_mm_alloc_t) mp_alloc,
-	};
-	knot_mm_t *pool = mm_alloc(&_pool, sizeof(*pool));
+	knot_mm_t *pool = mm_ctx_mempool2(MM_DEFAULT_BLKSIZE);
 	if (!pool) {
 		return NULL;
 	}
-	memcpy(pool, &_pool, sizeof(*pool));
-
 	struct io_stream_data *data = mm_alloc(pool, sizeof(struct io_stream_data));
 
 	data->buf = mp_start(pool->ctx, 512);
@@ -814,7 +808,11 @@ static void xdp_rx(uv_poll_t* handle, int status, int events)
 	assert(xhd && xhd->session && xhd->socket);
 	uint32_t rcvd;
 	knot_xdp_msg_t msgs[XDP_RX_BATCH_SIZE];
-	int ret = knot_xdp_recv(xhd->socket, msgs, XDP_RX_BATCH_SIZE, &rcvd);
+	int ret = knot_xdp_recv(xhd->socket, msgs, XDP_RX_BATCH_SIZE, &rcvd
+			#if KNOT_VERSION_HEX >= 0x030100
+			, NULL
+			#endif
+			);
 	if (ret == KNOT_EOK) {
 		kr_log_verbose("[xdp] poll triggered, processing a batch of %d packets\n",
 			(int)rcvd);
@@ -886,7 +884,12 @@ int io_listen_xdp(uv_loop_t *loop, struct endpoint *ep, const char *ifname)
 	xdp_handle_data_t *xhd = malloc(sizeof(*xhd));
 	if (!xhd) return kr_error(ENOMEM);
 
-	const int port = ep->port ? ep->port : KNOT_XDP_LISTEN_PORT_ALL;
+	const int port = ep->port ? ep->port : // all ports otherwise
+			#if KNOT_VERSION_HEX >= 0x030100
+				(KNOT_XDP_LISTEN_PORT_PASS | 0);
+			#else
+				KNOT_XDP_LISTEN_PORT_ALL;
+			#endif
 	xhd->socket = NULL; // needed for some reason
 	int ret = knot_xdp_init(&xhd->socket, ifname, ep->nic_queue, port,
 				KNOT_XDP_LOAD_BPF_MAYBE);
