@@ -1,4 +1,4 @@
-/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018-2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
  *  SPDX-License-Identifier: GPL-3.0-or-later
  */
 
@@ -17,17 +17,18 @@ struct session_flags {
 	bool outgoing : 1;      /**< True: to upstream; false: from a client. */
 	bool throttled : 1;     /**< True: data reading from peer is temporarily stopped. */
 	bool has_tls : 1;       /**< True: given session uses TLS. */
+	bool has_http : 1;      /**< True: given session uses HTTP. */
 	bool connected : 1;     /**< True: TCP connection is established. */
 	bool closing : 1;       /**< True: session close sequence is in progress. */
 	bool wirebuf_error : 1; /**< True: last operation with wirebuf ended up with an error. */
 };
 
-/* Allocate new session for a libuv handle.
- * If handle->tyoe is UV_UDP, tls parameter will be ignored. */
-struct session *session_new(uv_handle_t *handle, bool has_tls);
-/* Clear and free given session. */
+/** Allocate new session for a libuv handle.
+ * If handle->type isn't UV_TCP, has_* parameters will be ignored. */
+struct session *session_new(uv_handle_t *handle, bool has_tls, bool has_http);
+/** Clear and free given session. */
 void session_free(struct session *session);
-/* Clear session. */
+/** Clear session. */
 void session_clear(struct session *session);
 /** Close session. */
 void session_close(struct session *session);
@@ -84,17 +85,24 @@ struct sockaddr *session_get_peer(struct session *session);
 /** Get pointer to sockname (address of our end, not meaningful for UDP downstream). */
 struct sockaddr *session_get_sockname(struct session *session);
 /** Get pointer to server-side tls-related data. */
-struct tls_ctx_t *session_tls_get_server_ctx(const struct session *session);
+struct tls_ctx *session_tls_get_server_ctx(const struct session *session);
 /** Set pointer to server-side tls-related data. */
-void session_tls_set_server_ctx(struct session *session, struct tls_ctx_t *ctx);
+void session_tls_set_server_ctx(struct session *session, struct tls_ctx *ctx);
 /** Get pointer to client-side tls-related data. */
-struct tls_client_ctx_t *session_tls_get_client_ctx(const struct session *session);
+struct tls_client_ctx *session_tls_get_client_ctx(const struct session *session);
 /** Set pointer to client-side tls-related data. */
-void session_tls_set_client_ctx(struct session *session, struct tls_client_ctx_t *ctx);
-/** Get pointer to that part of tls-related data which has common structure for 
+void session_tls_set_client_ctx(struct session *session, struct tls_client_ctx *ctx);
+/** Get pointer to that part of tls-related data which has common structure for
  *  server and client. */
 struct tls_common_ctx *session_tls_get_common_ctx(const struct session *session);
 
+#if ENABLE_DOH2
+/** Get pointer to server-side http-related data. */
+struct http_ctx *session_http_get_server_ctx(const struct session *session);
+/** Set pointer to server-side http-related data. */
+void session_http_set_server_ctx(struct session *session, struct http_ctx *ctx);
+#endif
+
 /** Get pointer to underlying libuv handle for IO operations. */
 uv_handle_t *session_get_handle(struct session *session);
 struct session *session_get(uv_handle_t *h);
@@ -131,9 +139,9 @@ void session_unpoison(struct session *session);
 knot_pkt_t *session_produce_packet(struct session *session, knot_mm_t *mm);
 int session_discard_packet(struct session *session, const knot_pkt_t *pkt);
 
-void session_kill_ioreq(struct session *s, struct qr_task *task);
+void session_kill_ioreq(struct session *session, struct qr_task *task);
 /** Update timestamp */
-void session_touch(struct session *s);
+void session_touch(struct session *session);
 /** Returns either creation time or time of last IO activity if any occurs. */
 /* Used for TCP timeout calculation. */
-uint64_t session_last_activity(struct session *s);
+uint64_t session_last_activity(struct session *session);
