@@ -83,15 +83,15 @@ struct qr_task
 	qr_tasklist_t waiting;
 	struct session *pending[MAX_PENDING];
 	uint16_t pending_count;
-	uint16_t addrlist_count;
-	uint16_t addrlist_turn;
 	uint16_t timeouts;
 	uint16_t iter_count;
-	struct sockaddr *addrlist;
 	uint32_t refs;
 	bool finished : 1;
 	bool leading  : 1;
 	uint64_t creation_time;
+	uint64_t send_time;
+	uint64_t recv_time;
+	struct kr_transport *transport;
 };
 
 
@@ -120,15 +120,15 @@ static int qr_task_send(struct qr_task *task, struct session *session,
 			const struct sockaddr *addr, knot_pkt_t *pkt);
 static int qr_task_finalize(struct qr_task *task, int state);
 static void qr_task_complete(struct qr_task *task);
-static struct session* worker_find_tcp_connected(struct worker_ctx *worker,
+struct session* worker_find_tcp_connected(struct worker_ctx *worker,
 						 const struct sockaddr *addr);
 static int worker_add_tcp_waiting(struct worker_ctx *worker,
 				  const struct sockaddr *addr,
 				  struct session *session);
-static struct session* worker_find_tcp_waiting(struct worker_ctx *worker,
+struct session* worker_find_tcp_waiting(struct worker_ctx *worker,
 					       const struct sockaddr *addr);
 static void on_tcp_connect_timeout(uv_timer_t *timer);
-static void on_retransmit(uv_timer_t *req);
+static void on_udp_timeout(uv_timer_t *timer);
 static void subreq_finalize(struct qr_task *task, const struct sockaddr *packet_source, knot_pkt_t *pkt);
 
 
@@ -275,8 +275,13 @@ static uint8_t *alloc_wire_cb(struct kr_request *req, uint16_t *maxlen)
 	assert(handle->type == UV_POLL);
 	xdp_handle_data_t *xhd = handle->data;
 	knot_xdp_msg_t out;
-	int ret = knot_xdp_send_alloc(xhd->socket, ctx->source.addr.ip.sa_family == AF_INET6,
-					&out, NULL);
+	bool ipv6 = ctx->source.addr.ip.sa_family == AF_INET6;
+	int ret = knot_xdp_send_alloc(xhd->socket,
+			#if KNOT_VERSION_HEX >= 0x030100
+					ipv6 ? KNOT_XDP_MSG_IPV6 : 0, &out);
+			#else
+					ipv6, &out, NULL);
+			#endif
 	if (ret != KNOT_EOK) {
 		assert(ret == KNOT_ENOMEM);
 		*maxlen = 0;
@@ -310,6 +315,19 @@ static void free_wire(const struct request_ctx *ctx)
 	kr_log_verbose("[xdp] freed unsent buffer, ret = %d\n", ret);
 }
 #endif
+/* Helper functions for transport selection */
+static inline bool is_tls_capable(struct sockaddr *address) {
+	tls_client_param_t *tls_entry = tls_client_param_get(the_worker->engine->net.tls_client_params, address);
+	return tls_entry;
+}
+
+static inline bool is_tcp_connected(struct sockaddr *address) {
+	return worker_find_tcp_connected(the_worker, address);
+}
+
+static inline bool is_tcp_waiting(struct sockaddr *address) {
+	return worker_find_tcp_waiting(the_worker, address);
+}
 
 /** Create and initialize a request_ctx (on a fresh mempool).
  *
@@ -330,14 +348,12 @@ static struct request_ctx *request_create(struct worker_ctx *worker,
 	};
 
 	/* Create request context */
-	struct request_ctx *ctx = mm_alloc(&pool, sizeof(*ctx));
+	struct request_ctx *ctx = mm_calloc(&pool, 1, sizeof(*ctx));
 	if (!ctx) {
 		pool_release(worker, pool.ctx);
 		return NULL;
 	}
 
-	memset(ctx, 0, sizeof(*ctx));
-
 	/* TODO Relocate pool to struct request */
 	ctx->worker = worker;
 	if (session) {
@@ -383,6 +399,12 @@ static struct request_ctx *request_create(struct worker_ctx *worker,
 		req->qsource.dst_addr = &ctx->source.dst_addr.ip;
 	}
 
+	req->selection_context.is_tls_capable = is_tls_capable;
+	req->selection_context.is_tcp_connected = is_tcp_connected;
+	req->selection_context.is_tcp_waiting = is_tcp_waiting;
+	array_init(req->selection_context.forwarding_targets);
+	array_reserve_mm(req->selection_context.forwarding_targets, 1, kr_memreserve, &req->pool);
+
 	worker->stats.rconcurrent += 1;
 
 	return ctx;
@@ -415,10 +437,6 @@ static int request_start(struct request_ctx *ctx, knot_pkt_t *query)
 	struct engine *engine = worker->engine;
 	kr_resolve_begin(req, &engine->resolver);
 	worker->stats.queries += 1;
-	/* Throttle outbound queries only when high pressure */
-	if (worker->stats.concurrent < QUERY_RATE_THRESHOLD) {
-		req->options.NO_THROTTLE = true;
-	}
 	return kr_ok();
 }
 
@@ -465,11 +483,10 @@ static struct qr_task *qr_task_create(struct request_ctx *ctx)
 	}
 
 	/* Create resolution task */
-	struct qr_task *task = mm_alloc(&ctx->req.pool, sizeof(*task));
+	struct qr_task *task = mm_calloc(&ctx->req.pool, 1, sizeof(*task));
 	if (!task) {
 		return NULL;
 	}
-	memset(task, 0, sizeof(*task)); /* avoid accidentally unintialized fields */
 
 	/* Create packet buffers for answer and subrequests */
 	knot_pkt_t *pktbuf = knot_pkt_new(NULL, pktbuf_max, &ctx->req.pool);
@@ -559,7 +576,6 @@ static void qr_task_complete(struct qr_task *task)
 /* This is called when we send subrequest / answer */
 int qr_task_on_send(struct qr_task *task, const uv_handle_t *handle, int status)
 {
-
 	if (task->finished) {
 		assert(task->leading == false);
 		qr_task_complete(task);
@@ -572,26 +588,17 @@ int qr_task_on_send(struct qr_task *task, const uv_handle_t *handle, int status)
 	assert(s);
 
 	if (handle->type == UV_UDP && session_flags(s)->outgoing) {
-		/* Start the timeout timer for UDP here, since this is the closest
-		 * to the wire we can get. */
-		struct kr_request *req = &task->ctx->req;
-		/* Check current query NSLIST */
-		struct kr_query *qry = array_tail(req->rplan.pending);
+		// This should ensure that we are only dealing with our question to upstream
+		assert(!knot_wire_get_qr(task->pktbuf->wire));
+		// start the timer
+		struct kr_query *qry = array_tail(task->ctx->req.rplan.pending);
 		assert(qry != NULL);
-		/* Retransmit at default interval, or more frequently if the mean
-		 * RTT of the server is better. If the server is glued, use default rate. */
-		size_t timeout = qry->ns.score;
-		if (timeout > KR_NS_GLUED) {
-			/* We don't have information about variance in RTT, expect +10ms */
-			timeout = MIN(qry->ns.score + 10, KR_CONN_RETRY);
-		} else {
-			timeout = KR_CONN_RETRY;
-		}
-
-		int ret = session_timer_start(s, on_retransmit, timeout, 0);
+		(void)qry;
+		size_t timeout = task->transport->timeout;
+		int ret = session_timer_start(s, on_udp_timeout, timeout, 0);
 		/* Start next step with timeout, fatal if can't start a timer. */
 		if (ret != 0) {
-			subreq_finalize(task, &qry->ns.addr->ip, task->pktbuf);
+			subreq_finalize(task, &task->transport->address.ip, task->pktbuf);
 			qr_task_finalize(task, KR_STATE_FAIL);
 		}
 	}
@@ -681,6 +688,9 @@ static int qr_task_send(struct qr_task *task, struct session *session,
 	qr_task_ref(task);
 
 	struct worker_ctx *worker = ctx->worker;
+	/* Note time for upstream RTT */
+	task->send_time = kr_now();
+	task->recv_time = 0; // task structure is being reused so we have to zero this out here
 	/* Send using given protocol */
 	assert(!session_flags(session)->closing);
 	if (session_flags(session)->has_http) {
@@ -793,11 +803,9 @@ static int session_tls_hs_cb(struct session *session, int status)
 	if (status) {
 		struct qr_task *task = session_waitinglist_get(session);
 		if (task) {
-			struct kr_qflags *options = &task->ctx->req.options;
-			unsigned score = options->FORWARD || options->STUB ? KR_NS_FWD_DEAD : KR_NS_DEAD;
-			kr_nsrep_update_rtt(NULL, peer, score,
-					    the_worker->engine->resolver.cache_rtt,
-					    KR_NS_UPDATE_NORESET);
+			// TLS handshake failed, report it to server selection
+			struct kr_query *qry = array_tail(task->ctx->req.rplan.pending);
+			qry->server_selection.error(qry, task->transport, KR_SELECTION_TLS_HANDSHAKE_FAILED);
 		}
 #ifndef NDEBUG
 		else {
@@ -973,13 +981,10 @@ static void on_connect(uv_connect_t *req, int status)
 		struct qr_task *task = session_waitinglist_get(session);
 		if (task && status != UV_ETIMEDOUT) {
 			/* Penalize upstream.
-			 * In case of UV_ETIMEDOUT upstream has been
-			 * already penalized in on_tcp_connect_timeout() */
-			struct kr_qflags *options = &task->ctx->req.options;
-			unsigned score = options->FORWARD || options->STUB ? KR_NS_FWD_DEAD : KR_NS_DEAD;
-			kr_nsrep_update_rtt(NULL, peer, score,
-					    worker->engine->resolver.cache_rtt,
-					    KR_NS_UPDATE_NORESET);
+			* In case of UV_ETIMEDOUT upstream has been
+			* already penalized in on_tcp_connect_timeout() */
+			struct kr_query *qry = array_tail(task->ctx->req.rplan.pending);
+			qry->server_selection.error(qry, task->transport, KR_SELECTION_TCP_CONNECT_FAILED);
 		}
 		assert(session_tasklist_is_empty(session));
 		session_waitinglist_retry(session, false);
@@ -1061,10 +1066,7 @@ static void on_tcp_connect_timeout(uv_timer_t *timer)
 			    peer_str ? peer_str : "");
 	}
 
-	unsigned score = qry->flags.FORWARD || qry->flags.STUB ? KR_NS_FWD_DEAD : KR_NS_DEAD;
-	kr_nsrep_update_rtt(NULL, peer, score,
-			    worker->engine->resolver.cache_rtt,
-			    KR_NS_UPDATE_NORESET);
+	qry->server_selection.error(qry, task->transport, KR_SELECTION_TCP_CONNECT_TIMEOUT);
 
 	worker->stats.timeout += session_waitinglist_get_len(session);
 	session_waitinglist_retry(session, true);
@@ -1089,34 +1091,28 @@ static void on_udp_timeout(uv_timer_t *timer)
 
 	uv_timer_stop(timer);
 
-	/* Penalize all tried nameservers with a timeout. */
 	struct qr_task *task = session_tasklist_get_first(session);
 	struct worker_ctx *worker = task->ctx->worker;
+
 	if (task->leading && task->pending_count > 0) {
 		struct kr_query *qry = array_tail(task->ctx->req.rplan.pending);
-		struct sockaddr_in6 *addrlist = (struct sockaddr_in6 *)task->addrlist;
-		for (uint16_t i = 0; i < MIN(task->pending_count, task->addrlist_count); ++i) {
-			struct sockaddr *choice = (struct sockaddr *)(&addrlist[i]);
-			WITH_VERBOSE(qry) {
-				char *addr_str = kr_straddr(choice);
-				VERBOSE_MSG(qry, "=> server: '%s' flagged as 'bad'\n", addr_str ? addr_str : "");
-			}
-			unsigned score = qry->flags.FORWARD || qry->flags.STUB ? KR_NS_FWD_DEAD : KR_NS_DEAD;
-			kr_nsrep_update_rtt(&qry->ns, choice, score,
-					    worker->engine->resolver.cache_rtt,
-					    KR_NS_UPDATE_NORESET);
-		}
+		qry->server_selection.error(qry, task->transport, KR_SELECTION_QUERY_TIMEOUT);
 	}
+
 	task->timeouts += 1;
 	worker->stats.timeout += 1;
 	qr_task_step(task, NULL, NULL);
 }
 
-static uv_handle_t *retransmit(struct qr_task *task)
+static uv_handle_t *transmit(struct qr_task *task)
 {
 	uv_handle_t *ret = NULL;
-	if (task && task->addrlist && task->addrlist_count > 0) {
-		struct sockaddr_in6 *choice = &((struct sockaddr_in6 *)task->addrlist)[task->addrlist_turn];
+
+	if (task) {
+		struct kr_transport* transport = task->transport;
+
+		struct sockaddr_in6 *choice = (struct sockaddr_in6 *)&transport->address;
+
 		if (!choice) {
 			return ret;
 		}
@@ -1125,7 +1121,7 @@ static uv_handle_t *retransmit(struct qr_task *task)
 		}
 		/* Checkout answer before sending it */
 		struct request_ctx *ctx = task->ctx;
-		if (kr_resolve_checkout(&ctx->req, NULL, (struct sockaddr *)choice, SOCK_DGRAM, task->pktbuf) != 0) {
+		if (kr_resolve_checkout(&ctx->req, NULL, transport, task->pktbuf) != 0) {
 			return ret;
 		}
 		ret = ioreq_spawn(ctx->worker, SOCK_DGRAM, choice->sin6_family, false, false);
@@ -1144,31 +1140,12 @@ static uv_handle_t *retransmit(struct qr_task *task)
 		} else {
 			task->pending[task->pending_count] = session;
 			task->pending_count += 1;
-			task->addrlist_turn = (task->addrlist_turn + 1) %
-					       task->addrlist_count; /* Round robin */
 			session_start_read(session); /* Start reading answer */
 		}
 	}
 	return ret;
 }
 
-static void on_retransmit(uv_timer_t *req)
-{
-	struct session *session = req->data;
-	assert(session_tasklist_get_len(session) == 1);
-
-	uv_timer_stop(req);
-	struct qr_task *task = session_tasklist_get_first(session);
-	if (retransmit(task) == NULL) {
-		/* Not possible to spawn request, start timeout timer with remaining deadline. */
-		struct kr_qflags *options = &task->ctx->req.options;
-		uint64_t timeout = options->FORWARD || options->STUB ? KR_NS_FWD_TIMEOUT / 2 :
-				   KR_CONN_RTT_MAX - task->pending_count * KR_CONN_RETRY;
-		uv_timer_start(req, on_udp_timeout, timeout, 0);
-	} else {
-		uv_timer_start(req, on_retransmit, KR_CONN_RETRY, 0);
-	}
-}
 
 static void subreq_finalize(struct qr_task *task, const struct sockaddr *packet_source, knot_pkt_t *pkt)
 {
@@ -1196,6 +1173,12 @@ static void subreq_finalize(struct qr_task *task, const struct sockaddr *packet_
 			struct kr_query *qry = array_tail(follower->ctx->req.rplan.pending);
 			qry->id = leader_qry->id;
 			qry->secret = leader_qry->secret;
+
+			// Note that this transport may not be present in `leader_qry`'s server selection
+			follower->transport = task->transport;
+			if(follower->transport) {
+				follower->transport->deduplicated = true;
+			}
 			leader_qry->secret = 0; /* Next will be already decoded */
 		}
 		qr_task_step(follower, packet_source, pkt);
@@ -1369,7 +1352,7 @@ static int udp_task_step(struct qr_task *task,
 		return kr_ok(); /* Will be notified when outgoing query finishes. */
 	}
 	/* Start transmitting */
-	uv_handle_t *handle = retransmit(task);
+	uv_handle_t *handle = transmit(task);
 	if (handle == NULL) {
 		subreq_finalize(task, packet_source, packet);
 		return qr_task_finalize(task, KR_STATE_FAIL);
@@ -1517,15 +1500,7 @@ static int tcp_task_make_connection(struct qr_task *task, const struct sockaddr
 		worker_del_tcp_waiting(worker, addr);
 		free(conn);
 		session_close(session);
-		unsigned score = qry->flags.FORWARD || qry->flags.STUB ? KR_NS_FWD_DEAD : KR_NS_DEAD;
-		kr_nsrep_update_rtt(NULL, peer, score,
-				    worker->engine->resolver.cache_rtt,
-				    KR_NS_UPDATE_NORESET);
-		WITH_VERBOSE (qry) {
-			const char *peer_str = kr_straddr(peer);
-			kr_log_verbose( "[wrkr]=> connect to '%s' failed (%s), flagged as 'bad'\n",
-					peer_str ? peer_str : "", uv_strerror(ret));
-		}
+		qry->server_selection.error(qry, task->transport, KR_SELECTION_TCP_CONNECT_FAILED);
 		return kr_error(EAGAIN);
 	}
 
@@ -1549,7 +1524,7 @@ static int tcp_task_step(struct qr_task *task,
 	assert(task->pending_count == 0);
 
 	/* target */
-	const struct sockaddr *addr = task->addrlist;
+	const struct sockaddr *addr = &task->transport->address.ip;
 	if (addr->sa_family == AF_UNSPEC) {
 		/* Target isn't defined. Finalize task with SERVFAIL.
 		 * Although task->pending_count is zero, there are can be followers,
@@ -1559,8 +1534,7 @@ static int tcp_task_step(struct qr_task *task,
 	}
 	/* Checkout task before connecting */
 	struct request_ctx *ctx = task->ctx;
-	if (kr_resolve_checkout(&ctx->req, NULL, (struct sockaddr *)addr,
-				SOCK_STREAM, task->pktbuf) != 0) {
+	if (kr_resolve_checkout(&ctx->req, NULL, task->transport, task->pktbuf) != 0) {
 		subreq_finalize(task, packet_source, packet);
 		return qr_task_finalize(task, KR_STATE_FAIL);
 	}
@@ -1609,10 +1583,6 @@ static int qr_task_step(struct qr_task *task,
 	assert(ctx);
 	struct kr_request *req = &ctx->req;
 	struct worker_ctx *worker = ctx->worker;
-	int sock_type = -1;
-	task->addrlist = NULL;
-	task->addrlist_count = 0;
-	task->addrlist_turn = 0;
 
 	if (worker->too_many_open) {
 		/* */
@@ -1623,22 +1593,29 @@ static int qr_task_step(struct qr_task *task,
 		} else {
 			if (packet && kr_rplan_empty(rplan)) {
 				/* new query; TODO - make this detection more obvious */
-				kr_resolve_consume(req, packet_source, packet);
+				kr_resolve_consume(req, &task->transport, packet);
 			}
 			return qr_task_finalize(task, KR_STATE_FAIL);
 		}
 	}
 
-	int state = kr_resolve_consume(req, packet_source, packet);
+	// Report network RTT back to server selection
+	if (packet && task->send_time && task->recv_time) {
+		struct kr_query *qry = array_tail(req->rplan.pending);
+		qry->server_selection.update_rtt(qry, task->transport, task->recv_time - task->send_time);
+	}
+
+	int state = kr_resolve_consume(req, &task->transport, packet);
+
+	task->transport = NULL;
 	while (state == KR_STATE_PRODUCE) {
-		state = kr_resolve_produce(req, &task->addrlist,
-					   &sock_type, task->pktbuf);
+		state = kr_resolve_produce(req, &task->transport, task->pktbuf);
 		if (unlikely(++task->iter_count > KR_ITER_LIMIT ||
 			     task->timeouts >= KR_TIMEOUT_LIMIT)) {
 
 			#ifndef NOVERBOSELOG
 			struct kr_rplan *rplan = &req->rplan;
-			struct kr_query *last  = kr_rplan_last(rplan);
+			struct kr_query *last = kr_rplan_last(rplan);
 			if (task->iter_count > KR_ITER_LIMIT) {
 				VERBOSE_MSG(last, "canceling query due to exceeded iteration count limit of %d\n", KR_ITER_LIMIT);
 			}
@@ -1654,47 +1631,21 @@ static int qr_task_step(struct qr_task *task,
 	/* We're done, no more iterations needed */
 	if (state & (KR_STATE_DONE|KR_STATE_FAIL)) {
 		return qr_task_finalize(task, state);
-	} else if (!task->addrlist || sock_type < 0) {
+	} else if (!task->transport || !task->transport->protocol) {
 		return qr_task_step(task, NULL, NULL);
 	}
 
-	/* Count available address choices */
-	struct sockaddr_in6 *choice = (struct sockaddr_in6 *)task->addrlist;
-	for (size_t i = 0; i < KR_NSREP_MAXADDR && choice->sin6_family != AF_UNSPEC; ++i) {
-		task->addrlist_count += 1;
-		choice += 1;
+	switch (task->transport->protocol)
+	{
+	case KR_TRANSPORT_UDP:
+		return udp_task_step(task, packet_source, packet);
+	case KR_TRANSPORT_TCP: // fall through
+	case KR_TRANSPORT_TLS:
+		return tcp_task_step(task, packet_source, packet);
+	default:
+		assert(0);
+		return kr_error(EINVAL);
 	}
-
-	/* Upgrade to TLS if the upstream address is configured as DoT capable. */
-	if (task->addrlist_count > 0 && kr_inaddr_port(task->addrlist) == KR_DNS_PORT) {
-		/* TODO if there are multiple addresses (task->addrlist_count > 1)
-		 * check all of them. */
-		struct network *net = &worker->engine->net;
-		/* task->addrlist has to contain TLS port before tls_client_param_get() call */
-		kr_inaddr_set_port(task->addrlist, KR_DNS_TLS_PORT);
-		tls_client_param_t *tls_entry =
-			tls_client_param_get(net->tls_client_params, task->addrlist);
-		if (tls_entry) {
-			packet_source = NULL;
-			sock_type = SOCK_STREAM;
-			/* TODO in this case in tcp_task_make_connection() will be performed
-			 * redundant map_get() call. */
-		} else {
-			/* The function is fairly cheap, so we just change there and back. */
-			kr_inaddr_set_port(task->addrlist, KR_DNS_PORT);
-		}
-	}
-
-	int ret = 0;
-	if (sock_type == SOCK_DGRAM) {
-		/* Start fast retransmit with UDP. */
-		ret = udp_task_step(task, packet_source, packet);
-	} else {
-		/* TCP. Connect to upstream or send the query if connection already exists. */
-		assert (sock_type == SOCK_STREAM);
-		ret = tcp_task_step(task, packet_source, packet);
-	}
-	return ret;
 }
 
 static int parse_packet(knot_pkt_t *query)
@@ -1791,12 +1742,15 @@ int worker_submit(struct session *session,
 		}
 		assert(!session_flags(session)->closing);
 		addr = peer;
+		/* Note recieve time for RTT calculation */
+		task->recv_time = kr_now();
 	}
 	assert(uv_is_closing(session_get_handle(session)) == false);
 
 	/* Packet was successfully parsed.
 	 * Task was created (found). */
 	session_touch(session);
+
 	/* Consume input and produce next message */
 	return qr_task_step(task, addr, pkt);
 }
@@ -1851,7 +1805,7 @@ int worker_del_tcp_connected(struct worker_ctx *worker,
 	return map_del_tcp_session(&worker->tcp_connected, addr);
 }
 
-static struct session* worker_find_tcp_connected(struct worker_ctx *worker,
+struct session* worker_find_tcp_connected(struct worker_ctx *worker,
 						 const struct sockaddr* addr)
 {
 	return map_find_tcp_session(&worker->tcp_connected, addr);
@@ -1877,7 +1831,7 @@ int worker_del_tcp_waiting(struct worker_ctx *worker,
 	return map_del_tcp_session(&worker->tcp_waiting, addr);
 }
 
-static struct session* worker_find_tcp_waiting(struct worker_ctx *worker,
+struct session* worker_find_tcp_waiting(struct worker_ctx *worker,
 					       const struct sockaddr* addr)
 {
 	return map_find_tcp_session(&worker->tcp_waiting, addr);
@@ -1951,12 +1905,9 @@ int worker_end_tcp(struct session *session)
 	return kr_ok();
 }
 
-knot_pkt_t * worker_resolve_mk_pkt(const char *qname_str, uint16_t qtype, uint16_t qclass,
+knot_pkt_t *worker_resolve_mk_pkt_dname(knot_dname_t *qname, uint16_t qtype, uint16_t qclass,
 				   const struct kr_qflags *options)
 {
-	uint8_t qname[KNOT_DNAME_MAXLEN];
-	if (!knot_dname_from_str(qname, qname_str, sizeof(qname)))
-		return NULL;
 	knot_pkt_t *pkt = knot_pkt_new(NULL, KNOT_EDNS_MAX_UDP_PAYLOAD, NULL);
 	if (!pkt)
 		return NULL;
@@ -1991,6 +1942,15 @@ knot_pkt_t * worker_resolve_mk_pkt(const char *qname_str, uint16_t qtype, uint16
 	return pkt;
 }
 
+knot_pkt_t *worker_resolve_mk_pkt(const char *qname_str, uint16_t qtype, uint16_t qclass,
+				   const struct kr_qflags *options)
+{
+	uint8_t qname[KNOT_DNAME_MAXLEN];
+	if (!knot_dname_from_str(qname, qname_str, sizeof(qname)))
+		return NULL;
+	return worker_resolve_mk_pkt_dname(qname, qtype, qclass, options);
+}
+
 struct qr_task *worker_resolve_start(knot_pkt_t *query, struct kr_qflags options)
 {
 	struct worker_ctx *worker = the_worker;
@@ -2149,8 +2109,7 @@ static int worker_reserve(struct worker_ctx *worker, size_t ring_maxlen)
 		return kr_error(ENOMEM);
 	}
 
-	worker->pkt_pool.ctx = mp_new (4 * sizeof(knot_pkt_t));
-	worker->pkt_pool.alloc = (knot_mm_alloc_t) mp_alloc;
+	mm_ctx_mempool(&worker->pkt_pool, 4 * sizeof(knot_pkt_t));
 
 	return kr_ok();
 }
