@@ -47,6 +47,9 @@ int worker_submit(struct session *session,
  */
 int worker_end_tcp(struct session *session);
 
+KR_EXPORT knot_pkt_t *worker_resolve_mk_pkt_dname(knot_dname_t *qname, uint16_t qtype, uint16_t qclass,
+				   const struct kr_qflags *options);
+
 /**
  * Create a packet suitable for worker_resolve_start().  All in malloc() memory.
  */
@@ -96,6 +99,10 @@ int worker_del_tcp_connected(struct worker_ctx *worker,
 			     const struct sockaddr *addr);
 int worker_del_tcp_waiting(struct worker_ctx *worker,
 			   const struct sockaddr* addr);
+struct session* worker_find_tcp_waiting(struct worker_ctx *worker,
+					       const struct sockaddr* addr);
+struct session* worker_find_tcp_connected(struct worker_ctx *worker,
+					       const struct sockaddr* addr);
 knot_pkt_t *worker_task_get_pktbuf(const struct qr_task *task);
 
 struct request_ctx *worker_task_get_request(struct qr_task *task);
@@ -136,7 +143,7 @@ struct worker_stats {
 /** @cond internal */
 
 /** Number of request within timeout window. */
-#define MAX_PENDING KR_NSREP_MAXADDR
+#define MAX_PENDING 4
 
 /** Maximum response time from TCP upstream, milliseconds */
 #define MAX_TCP_INACTIVITY (KR_RESOLVE_TIME_LIMIT + KR_CONN_RTT_MAX)
