@@ -83,3 +83,13 @@ man_pages = [
     ('index', 'libkres', u'libkres documentation',
      [u'CZ.NIC Labs'], 1)
 ]
+
+# -- Options for Texinfo output ------------------------------------------------
+
+# Grouping the document tree into Texinfo files. List of tuples
+# (source start file, target name, title, author,
+#  dir menu entry, description, category)
+texinfo_documents = [
+    ('index', 'knot-resolver', u'Knot Resolver', u'CZ.NIC Labs',
+     'Knot Resolver', 'Caching DNS resolver.', 'Network services'),
+]
