@@ -1,17 +1,5 @@
 /*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ *  SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 /** @file
@@ -26,7 +14,8 @@
 
 
 /** Compute TTL for a packet.  Generally it's minimum TTL, with extra conditions. */
-static uint32_t packet_ttl(const knot_pkt_t *pkt, bool is_negative)
+KR_EXPORT
+uint32_t packet_ttl(const knot_pkt_t *pkt, bool is_negative)
 {
 	bool has_ttl = false;
 	uint32_t ttl = UINT32_MAX;
@@ -47,6 +36,7 @@ static uint32_t packet_ttl(const knot_pkt_t *pkt, bool is_negative)
 				continue; /* Skip metatypes. */
 			}
 			ttl = MIN(ttl, rr->ttl);
+			has_ttl = true;
 		}
 	}
 	/* If no valid TTL present, go with zero (will get clamped to minimum). */
@@ -55,15 +45,15 @@ static uint32_t packet_ttl(const knot_pkt_t *pkt, bool is_negative)
 
 
 void stash_pkt(const knot_pkt_t *pkt, const struct kr_query *qry,
-		const struct kr_request *req, const bool has_optout)
+		const struct kr_request *req, const bool needs_pkt)
 {
 	/* In some cases, stash also the packet. */
 	const bool is_negative = kr_response_classify(pkt)
 				& (PKT_NODATA|PKT_NXDOMAIN);
 	const struct kr_qflags * const qf = &qry->flags;
-	const bool want_negative = qf->DNSSEC_INSECURE || !qf->DNSSEC_WANT || has_optout;
+	const bool want_negative = qf->DNSSEC_INSECURE || !qf->DNSSEC_WANT;
 	const bool want_pkt = qf->DNSSEC_BOGUS /*< useful for +cd answers */
-				|| (is_negative && want_negative);
+				|| (is_negative && want_negative) || needs_pkt;
 
 	if (!want_pkt || !knot_wire_get_aa(pkt->wire)
 	    || pkt->parsed != pkt->size /*< malformed packet; still can't detect KNOT_EFEWDATA */
@@ -87,7 +77,7 @@ void stash_pkt(const knot_pkt_t *pkt, const struct kr_query *qry,
 			kr_rank_set(&rank, KR_RANK_INSECURE);
 		} else if (!qf->DNSSEC_WANT) {
 			/* no TAs at all, leave _RANK_AUTH */
-		} else if (has_optout) {
+		} else if (needs_pkt) {
 			/* All bad cases should be filtered above,
 			 * at least the same way as pktcache in kresd 1.5.x. */
 			kr_rank_set(&rank, KR_RANK_SECURE);
@@ -155,14 +145,22 @@ int answer_from_pkt(kr_layer_t *ctx, knot_pkt_t *pkt, uint16_t type,
 	struct kr_request *req = ctx->req;
 	struct kr_query *qry = req->current_query;
 
+	const uint16_t msgid = knot_wire_get_id(pkt->wire);
+
+	/* Ensure the wire buffer is large enough.  Strategy: fit and at least double. */
 	uint16_t pkt_len;
 	memcpy(&pkt_len, eh->data, sizeof(pkt_len));
 	if (pkt_len > pkt->max_size) {
-		return kr_error(ENOENT);
+		pkt->max_size = MIN(KNOT_WIRE_MAX_PKTSIZE,
+				    MAX(pkt->max_size * 2, pkt_len));
+		mm_free(&ctx->req->pool, pkt->wire); /* no-op, but... */
+		pkt->wire = mm_alloc(&ctx->req->pool, pkt->max_size);
+		pkt->compr.wire = pkt->wire;
+		/* TODO: ^^ nicer way how to replace knot_pkt_t::wire ? */
 	}
+	assert(pkt->max_size >= pkt_len);
 
 	/* Copy answer and reparse it, but keep the original message id. */
-	uint16_t msgid = knot_wire_get_id(pkt->wire);
 	knot_pkt_clear(pkt);
 	memcpy(pkt->wire, eh->data + 2, pkt_len);
 	pkt->size = pkt_len;
