@@ -1,17 +1,5 @@
 /*  Copyright (C) 2016-2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  SPDX-License-Identifier: GPL-3.0-or-later
 
     The code originated from https://github.com/fanf2/qp/blob/master/qp.c
     at revision 5f6d93753.
@@ -216,8 +204,8 @@ static node_t* twig(node_t *t, uint i)
  * Having this separate might be meaningful for performance optimization.
  */
 #define TWIGOFFMAX(off, max, t, b) do {			\
-		off = twigoff(t, b);			\
-		max = bitmap_weight(t->branch.bitmap);	\
+		(off) = twigoff((t), (b));		\
+		(max) = bitmap_weight((t)->branch.bitmap);\
 	} while(0)
 
 /*! \brief Simple string comparator. */
