@@ -29,7 +29,7 @@
 #include "lib/resolve.h"
 #include "lib/rplan.h"
 #include "lib/defines.h"
-#include "lib/nsrep.h"
+#include "lib/selection.h"
 #include "lib/module.h"
 #include "lib/dnssec/ta.h"
 
@@ -98,7 +98,7 @@ static bool is_authoritative(const knot_pkt_t *answer, struct kr_query *query)
 	}
 
 #ifndef STRICT_MODE
-	/* Last resort to work around broken auths, if the zone cut is at/parent of the QNAME. */
+	/* Last resort to work around broken auths, if the zone cut is at the QNAME. */
 	if (knot_dname_is_equal(query->zone_cut.name, knot_pkt_qname(answer))) {
 		return true;
 	}
@@ -213,10 +213,12 @@ static void fetch_glue(knot_pkt_t *pkt, const knot_dname_t *ns, bool in_bailiwic
 
 			if ((rr->type == KNOT_RRTYPE_A) &&
 			    (req->ctx->options.NO_IPV4)) {
+				QVERBOSE_MSG(qry, "<= skipping IPv4 glue due to network settings\n");
 				continue;
 			}
 			if ((rr->type == KNOT_RRTYPE_AAAA) &&
 			    (req->ctx->options.NO_IPV6)) {
+				QVERBOSE_MSG(qry, "<= skipping IPv6 glue due to network settings\n");
 				continue;
 			}
 			(void) update_nsaddr(rr, req->current_query, glue_cnt);
@@ -258,6 +260,7 @@ static int update_cut(knot_pkt_t *pkt, const knot_rrset_t *rr,
 		     && knot_dname_in_bailiwick(qry->sname, rr->owner)  >= 0;
 	if (!ok) {
 		VERBOSE_MSG("<= authority: ns outside bailiwick\n");
+		qry->server_selection.error(qry, req->upstream.transport, KR_SELECTION_LAME_DELEGATION);
 #ifdef STRICT_MODE
 		return KR_STATE_FAIL;
 #else
@@ -632,10 +635,11 @@ static int process_referral_answer(knot_pkt_t *pkt, struct kr_request *req)
 {
 	const knot_dname_t *cname = NULL;
 	int state = unroll_cname(pkt, req, true, &cname);
+	struct kr_query *query = req->current_query;
 	if (state != kr_ok()) {
+		query->server_selection.error(query, req->upstream.transport, KR_SELECTION_BAD_CNAME);
 		return KR_STATE_FAIL;
 	}
-	struct kr_query *query = req->current_query;
 	if (!(query->flags.CACHED)) {
 		/* If not cached (i.e. got from upstream)
 		 * make sure that this is not an authoritative answer
@@ -721,6 +725,7 @@ static int process_answer(knot_pkt_t *pkt, struct kr_request *req)
 	if (!is_authoritative(pkt, query)) {
 		if (!(query->flags.FORWARD) &&
 		    pkt_class & (PKT_NXDOMAIN|PKT_NODATA)) {
+			query->server_selection.error(query, req->upstream.transport, KR_SELECTION_LAME_DELEGATION);
 			VERBOSE_MSG("<= lame response: non-auth sent negative response\n");
 			return KR_STATE_FAIL;
 		}
@@ -730,6 +735,7 @@ static int process_answer(knot_pkt_t *pkt, struct kr_request *req)
 	/* Process answer type */
 	int state = unroll_cname(pkt, req, false, &cname);
 	if (state != kr_ok()) {
+		query->server_selection.error(query, req->upstream.transport, KR_SELECTION_BAD_CNAME);
 		return state;
 	}
 	/* Make sure that this is an authoritative answer (even with AA=0) for other layers */
@@ -760,6 +766,7 @@ static int process_answer(knot_pkt_t *pkt, struct kr_request *req)
 			    q->stype == query->stype   &&
 			    knot_dname_is_equal(q->sname, cname)) {
 				VERBOSE_MSG("<= cname chain loop\n");
+				query->server_selection.error(query, req->upstream.transport, KR_SELECTION_BAD_CNAME);
 				return KR_STATE_FAIL;
 			}
 		}
@@ -772,17 +779,12 @@ static int process_answer(knot_pkt_t *pkt, struct kr_request *req)
 		/* Copy transitive flags from original query to CNAME followup. */
 		next->flags.TRACE = query->flags.TRACE;
 		next->flags.ALWAYS_CUT = query->flags.ALWAYS_CUT;
-		next->flags.NO_MINIMIZE = query->flags.NO_MINIMIZE;
-		next->flags.NO_THROTTLE = query->flags.NO_THROTTLE;
+
+		/* Original query might have turned minimization off, revert. */
+		next->flags.NO_MINIMIZE = req->options.NO_MINIMIZE;
 
 		if (query->flags.FORWARD) {
 			next->forward_flags.CNAME = true;
-			if (query->parent == NULL) {
-				state = kr_nsrep_copy_set(&next->ns, &query->ns);
-				if (state != kr_ok()) {
-					return KR_STATE_FAIL;
-				}
-			}
 		}
 		next->cname_parent = query;
 		/* Want DNSSEC if and only if it's posible to secure
@@ -872,14 +874,6 @@ static int process_stub(knot_pkt_t *pkt, struct kr_request *req)
 	return finalize_answer(pkt, req);
 }
 
-
-/** Error handling, RFC1034 5.3.3, 4d.
- * NOTE: returing this does not prevent further queries (by itself). */
-static int resolve_error(knot_pkt_t *pkt, struct kr_request *req)
-{
-	return KR_STATE_FAIL;
-}
-
 /* State-less single resolution iteration step, not needed. */
 static int reset(kr_layer_t *ctx)  { return KR_STATE_PRODUCE; }
 
@@ -973,23 +967,22 @@ static int prepare_query(kr_layer_t *ctx, knot_pkt_t *pkt)
 	return KR_STATE_CONSUME;
 }
 
-static int resolve_badmsg(knot_pkt_t *pkt, struct kr_request *req, struct kr_query *query)
+static bool satisfied_by_additional(const struct kr_query *qry)
 {
-
-#ifndef STRICT_MODE
-	/* Work around broken auths/load balancers */
-	if (query->flags.SAFEMODE) {
-		return resolve_error(pkt, req);
-	} else if (query->flags.NO_MINIMIZE) {
-		query->flags.SAFEMODE = true;
-		return KR_STATE_DONE;
-	} else {
-		query->flags.NO_MINIMIZE = true;
-		return KR_STATE_DONE;
+	const bool prereq = !qry->flags.STUB && !qry->flags.FORWARD && qry->flags.NONAUTH;
+	if (!prereq)
+		return false;
+	const struct kr_request *req = qry->request;
+	for (ssize_t i = req->add_selected.len - 1; i >= 0; --i) {
+		ranked_rr_array_entry_t *entry = req->add_selected.at[i];
+		if (entry->qry_uid != qry->uid)
+			break;
+		if (entry->rr->type == qry->stype
+		    && knot_dname_is_equal(entry->rr->owner, qry->sname)) {
+			return true;
+		}
 	}
-#else
-		return resolve_error(pkt, req);
-#endif
+	return false;
 }
 
 /** Resolve input query or continue resolution with followups.
@@ -1023,32 +1016,39 @@ static int resolve(kr_layer_t *ctx, knot_pkt_t *pkt)
 #ifdef STRICT_MODE
 	if (pkt->parsed < pkt->size) {
 		VERBOSE_MSG("<= pkt contains excessive data\n");
-		return resolve_badmsg(pkt, req, query);
+		return KR_STATE_FAIL;
 	} else
 #endif
 	if (pkt->parsed <= KNOT_WIRE_HEADER_SIZE) {
+		if (pkt->parsed == KNOT_WIRE_HEADER_SIZE && knot_wire_get_rcode(pkt->wire) == KNOT_RCODE_FORMERR) {
+			/* This is a special case where we get valid header with FORMERR and nothing else.
+			 * This happens on some authoritatives which don't support EDNS and don't
+			 * bother copying the SECTION QUESTION. */
+			query->server_selection.error(query, req->upstream.transport, KR_SELECTION_FORMERR);
+			return KR_STATE_FAIL;
+		}
 		VERBOSE_MSG("<= malformed response (parsed %d)\n", (int)pkt->parsed);
-		return resolve_badmsg(pkt, req, query);
+		query->server_selection.error(query, req->upstream.transport, KR_SELECTION_MALFORMED);
+		return KR_STATE_FAIL;
 	} else if (!is_paired_to_query(pkt, query)) {
 		WITH_VERBOSE(query) {
 			const char *ns_str =
-				req->upstream.addr ? kr_straddr(req->upstream.addr) : "(internal)";
+				req->upstream.transport ? kr_straddr(&req->upstream.transport->address.ip) : "(internal)";
 			VERBOSE_MSG("<= ignoring mismatching response from %s\n",
 					ns_str ? ns_str : "(kr_straddr failed)");
 		}
-		/* Force TCP, to work around authoritatives messing up question
-		 * without yielding to spoofed responses. */
-		query->flags.TCP = true;
-		return resolve_badmsg(pkt, req, query);
+		query->server_selection.error(query, req->upstream.transport, KR_SELECTION_MISMATCHED);
+		return KR_STATE_FAIL;
 	} else if (knot_wire_get_tc(pkt->wire)) {
 		VERBOSE_MSG("<= truncated response, failover to TCP\n");
 		if (query) {
 			/* Fail if already on TCP. */
-			if (query->flags.TCP) {
+			if (req->upstream.transport->protocol != KR_TRANSPORT_UDP) {
 				VERBOSE_MSG("<= TC=1 with TCP, bailing out\n");
-				return resolve_error(pkt, req);
+				query->server_selection.error(query, req->upstream.transport, KR_SELECTION_TRUNCATED);
+				return KR_STATE_FAIL;
 			}
-			query->flags.TCP = true;
+			query->server_selection.error(query, req->upstream.transport, KR_SELECTION_TRUNCATED);
 		}
 		return KR_STATE_CONSUME;
 	}
@@ -1061,30 +1061,62 @@ static int resolve(kr_layer_t *ctx, knot_pkt_t *pkt)
 	const knot_lookup_t *rcode = knot_lookup_by_id(knot_rcode_names, knot_wire_get_rcode(pkt->wire));
 #endif
 
+	// We can't return directly from the switch because we have to give feedback to server selection first
+	int ret = 0;
+	int selection_error = KR_SELECTION_OK;
+
 	/* Check response code. */
 	switch(knot_wire_get_rcode(pkt->wire)) {
 	case KNOT_RCODE_NOERROR:
 	case KNOT_RCODE_NXDOMAIN:
 		break; /* OK */
 	case KNOT_RCODE_YXDOMAIN: /* Basically a successful answer; name just doesn't fit. */
-		if (!kr_request_ensure_answer(req))
-			return req->state;
+		if (!kr_request_ensure_answer(req)) {
+			ret = req->state;
+		}
 		knot_wire_set_rcode(req->answer->wire, KNOT_RCODE_YXDOMAIN);
 		break;
 	case KNOT_RCODE_REFUSED:
+		if (query->flags.STUB) {
+			 /* just pass answer through if in stub mode */
+			break;
+		}
+		ret = KR_STATE_FAIL;
+		selection_error = KR_SELECTION_REFUSED;
+		break;
 	case KNOT_RCODE_SERVFAIL:
 		if (query->flags.STUB) {
 			 /* just pass answer through if in stub mode */
 			break;
 		}
-		/* fall through */
+		ret = KR_STATE_FAIL;
+		selection_error = KR_SELECTION_SERVFAIL;
+		break;
 	case KNOT_RCODE_FORMERR:
+		ret = KR_STATE_FAIL;
+		if (knot_pkt_has_edns(pkt)) {
+			selection_error = KR_SELECTION_FORMERR_EDNS;
+		} else {
+			selection_error = KR_SELECTION_FORMERR;
+		}
+		break;
 	case KNOT_RCODE_NOTIMPL:
-		VERBOSE_MSG("<= rcode: %s\n", rcode ? rcode->name : "??");
-		return resolve_badmsg(pkt, req, query);
+		ret = KR_STATE_FAIL;
+		selection_error = KR_SELECTION_NOTIMPL;
+		break;
 	default:
+		ret = KR_STATE_FAIL;
+		selection_error = KR_SELECTION_OTHER_RCODE;
+		break;
+	}
+
+	if (query->server_selection.initialized) {
+		query->server_selection.error(query, req->upstream.transport, selection_error);
+	}
+
+	if (ret) {
 		VERBOSE_MSG("<= rcode: %s\n", rcode ? rcode->name : "??");
-		return resolve_error(pkt, req);
+		return ret;
 	}
 
 	int state;
@@ -1103,7 +1135,19 @@ static int resolve(kr_layer_t *ctx, knot_pkt_t *pkt)
 		break;
 	case KR_STATE_DONE: /* Referral */
 		state = process_referral_answer(pkt,req);
-		VERBOSE_MSG("<= referral response, follow\n");
+		if (satisfied_by_additional(query)) { /* This is a little hacky.
+			 * We found sufficient information in ADDITIONAL section
+			 * and it was selected for caching in this CONSUME round.
+			 * To make iterator accept the record in a simple way,
+			 * we trigger another cache *reading* attempt
+			 * for the subsequent PRODUCE round.
+			 */
+			assert(query->flags.NONAUTH);
+			query->flags.CACHE_TRIED = false;
+			VERBOSE_MSG("<= referral response, but cache should stop us short now\n");
+		} else {
+			VERBOSE_MSG("<= referral response, follow\n");
+		}
 		break;
 	default:
 		break;
@@ -1115,7 +1159,7 @@ rrarray_finalize:
 	(void)0;
 	ranked_rr_array_t *selected[] = kr_request_selected(req);
 	for (knot_section_t i = KNOT_ANSWER; i <= KNOT_ADDITIONAL; ++i) {
-		int ret = kr_ranked_rrarray_finalize(selected[i], query->uid, &req->pool);
+		ret = kr_ranked_rrarray_finalize(selected[i], query->uid, &req->pool);
 		if (unlikely(ret)) {
 			return KR_STATE_FAIL;
 		}
