@@ -69,12 +69,11 @@ void kr_qflags_clear(struct kr_qflags *fl1, struct kr_qflags fl2)
 
 static struct kr_query *query_create(knot_mm_t *pool, const knot_dname_t *name, uint32_t uid)
 {
-	struct kr_query *qry = mm_alloc(pool, sizeof(struct kr_query));
+	struct kr_query *qry = mm_calloc(pool, 1, sizeof(*qry));
 	if (qry == NULL) {
 		return NULL;
 	}
 
-	memset(qry, 0, sizeof(struct kr_query));
 	if (name != NULL) {
 		qry->sname = knot_dname_copy(name, pool);
 		if (qry->sname == NULL) {
@@ -159,22 +158,13 @@ static struct kr_query *kr_rplan_push_query(struct kr_rplan *rplan,
 	qry->flags = rplan->request->options;
 	qry->parent = parent;
 	qry->request = rplan->request;
-	qry->ns.ctx = rplan->request->ctx;
-	qry->ns.addr[0].ip.sa_family = AF_UNSPEC;
+
 	gettimeofday(&qry->timestamp, NULL);
 	qry->timestamp_mono = kr_now();
 	qry->creation_time_mono = parent ? parent->creation_time_mono : qry->timestamp_mono;
 	kr_zonecut_init(&qry->zone_cut, (const uint8_t *)"", rplan->pool);
 	qry->reorder = qry->flags.REORDER_RR ? kr_rand_bytes(sizeof(qry->reorder)) : 0;
 
-	/* When forwarding, keep the nameserver addresses. */
-	if (parent && parent->flags.FORWARD && qry->flags.FORWARD) {
-		ret = kr_nsrep_copy_set(&qry->ns, &parent->ns);
-		if (ret) {
-			query_free(rplan->pool, qry);
-			return NULL;
-		}
-	}
 
 	assert((rplan->pending.len == 0 && rplan->resolved.len == 0)
 		== (rplan->initial == NULL));
