@@ -8,26 +8,25 @@
 #include <libknot/dname.h>
 #include <libknot/codes.h>
 
+#include "lib/selection.h"
 #include "lib/cache/api.h"
 #include "lib/zonecut.h"
-#include "lib/nsrep.h"
 
 /** Query flags */
 struct kr_qflags {
 	bool NO_MINIMIZE : 1;    /**< Don't minimize QNAME. */
-	bool NO_THROTTLE : 1;    /**< No query/slow NS throttling. */
 	bool NO_IPV6 : 1;        /**< Disable IPv6 */
 	bool NO_IPV4 : 1;        /**< Disable IPv4 */
-	bool TCP : 1;            /**< Use TCP for this query. */
+	bool TCP : 1;            /**< Use TCP (or TLS) for this query. */
 	bool RESOLVED : 1;       /**< Query is resolved.  Note that kr_query gets
 				  *   RESOLVED before following a CNAME chain; see .CNAME. */
 	bool AWAIT_IPV4 : 1;     /**< Query is waiting for A address. */
 	bool AWAIT_IPV6 : 1;     /**< Query is waiting for AAAA address. */
 	bool AWAIT_CUT : 1;      /**< Query is waiting for zone cut lookup */
-	bool SAFEMODE : 1;       /**< Don't use fancy stuff (EDNS, 0x20, ...) */
+	bool NO_EDNS : 1;        /**< Don't use EDNS. */
 	bool CACHED : 1;         /**< Query response is cached. */
 	bool NO_CACHE : 1;       /**< No cache for lookup; exception: finding NSs and subqueries. */
-	bool EXPIRING : 1;       /**< Query response is cached, but expiring. */
+	bool EXPIRING : 1;       /**< Query response is cached but expiring.  See is_expiring(). */
 	bool ALLOW_LOCAL : 1;    /**< Allow queries to local or private address ranges. */
 	bool DNSSEC_WANT : 1;    /**< Want DNSSEC secured answer; exception: +cd,
 				  * i.e. knot_wire_get_cd(request->qsource.packet->wire) */
@@ -101,8 +100,7 @@ struct kr_query {
 	struct kr_query *cname_parent;
 	struct kr_request *request; /**< Parent resolution request. */
 	kr_stale_cb stale_cb; /**< See the type */
-	/* Beware: this must remain the last, because of lua bindings. */
-	struct kr_nsrep ns;
+	struct kr_server_selection server_selection;
 };
 
 /** @cond internal Array of queries. */
