@@ -11,7 +11,7 @@
 #include "lib/defines.h"
 #include "lib/generic/array.h"
 #include "lib/module.h"
-#include "lib/nsrep.h"
+#include "lib/selection.h"
 #include "lib/resolve.h"
 
 #include <gnutls/gnutls.h>
@@ -39,43 +39,6 @@
 /* Logging & debugging */
 bool kr_verbose_status = false;
 
-void *mm_realloc(knot_mm_t *mm, void *what, size_t size, size_t prev_size)
-{
-	if (mm) {
-		void *p = mm->alloc(mm->ctx, size);
-		if (p == NULL) {
-			return NULL;
-		} else {
-			if (what) {
-				memcpy(p, what,
-				       prev_size < size ? prev_size : size);
-			}
-			mm_free(mm, what);
-			return p;
-		}
-	} else {
-		return realloc(what, size);
-	}
-}
-
-void *mm_malloc(void *ctx, size_t n)
-{
-	(void)ctx;
-	return malloc(n);
-}
-void *mm_malloc_aligned(void *ctx, size_t n)
-{
-	size_t alignment = (size_t)ctx;
-	void *res;
-	int err = posix_memalign(&res, alignment, n);
-	if (err == 0) {
-		return res;
-	} else {
-		assert(err == -1 && errno == ENOMEM);
-		return NULL;
-	}
-}
-
 /*
  * Macros.
  */
@@ -536,7 +499,7 @@ int kr_straddr_subnet(void *dst, const char *addr)
 	}
 	/* Parse address */
 	int ret = inet_pton(family, addr_str, dst);
-	if (ret < 0) {
+	if (ret != 1) {
 		return kr_error(EILSEQ);
 	}
 
@@ -577,7 +540,7 @@ int kr_straddr_join(const char *addr, uint16_t port, char *buf, size_t *buflen)
 
 	struct sockaddr_storage ss;
 	int family = kr_straddr_family(addr);
-	if (family == kr_error(EINVAL) || !inet_pton(family, addr, &ss)) {
+	if (family == kr_error(EINVAL) || inet_pton(family, addr, &ss) != 1) {
 		return kr_error(EINVAL);
 	}
 
@@ -789,11 +752,10 @@ int kr_ranked_rrarray_add(ranked_rr_array_t *array, const knot_rrset_t *rr,
 		return kr_error(ret);
 	}
 
-	ranked_rr_array_entry_t *entry = mm_alloc(pool, sizeof(ranked_rr_array_entry_t));
+	ranked_rr_array_entry_t *entry = mm_calloc(pool, 1, sizeof(*entry));
 	if (!entry) {
 		return kr_error(ENOMEM);
 	}
-	memset(entry, 0, sizeof(*entry)); /* default all to zeros */
 
 	knot_rrset_t *rr_new = knot_rrset_new(rr->owner, rr->type, rr->rclass, rr->ttl, pool);
 	if (!rr_new) {
@@ -862,19 +824,16 @@ int kr_ranked_rrarray_finalize(ranked_rr_array_t *array, uint32_t qry_uid, knot_
 				}
 			}
 			/* Prepare rdataset, except rdata contents. */
-			int size_sum = 0;
+			knot_rdataset_t *rds = &stashed->rr->rrs;
+			rds->size = 0;
 			for (int i = 0; i < ra->len; ++i) {
 				if (ra->at[i]) {
-					size_sum += knot_rdata_size(ra->at[i]->len);
+					rds->size += knot_rdata_size(ra->at[i]->len);
 				}
 			}
-			knot_rdataset_t *rds = &stashed->rr->rrs;
 			rds->count = ra->len - dup_count;
-			#if KNOT_VERSION_HEX >= 0x020900
-				rds->size = size_sum;
-			#endif
-			if (size_sum) {
-				rds->rdata = mm_alloc(pool, size_sum);
+			if (rds->size) {
+				rds->rdata = mm_alloc(pool, rds->size);
 				if (!rds->rdata) {
 					return kr_error(ENOMEM);
 				}
@@ -884,13 +843,13 @@ int kr_ranked_rrarray_finalize(ranked_rr_array_t *array, uint32_t qry_uid, knot_
 			/* Everything is ready; now just copy all the rdata. */
 			uint8_t *raw_it = (uint8_t *)rds->rdata;
 			for (int i = 0; i < ra->len; ++i) {
-				if (ra->at[i] && size_sum/*linters*/) {
+				if (ra->at[i] && rds->size/*linters*/) {
 					const int size = knot_rdata_size(ra->at[i]->len);
 					memcpy(raw_it, ra->at[i], size);
 					raw_it += size;
 				}
 			}
-			assert(raw_it == (uint8_t *)rds->rdata + size_sum);
+			assert(raw_it == (uint8_t *)rds->rdata + rds->size);
 		}
 		stashed->in_progress = false;
 	}
