@@ -23,8 +23,9 @@
 #include <uv.h>
 
 #include "kresconfig.h"
-#include "lib/generic/array.h"
+#include "contrib/mempattern.h"
 #include "lib/defines.h"
+#include "lib/generic/array.h"
 
 struct kr_query;
 struct kr_request;
@@ -110,59 +111,8 @@ void kr_log_q(const struct kr_query *qry, const char *source, const char *fmt, .
 #define static_assert(cond, msg)
 #endif
 
-/** @cond Memory alloc routines */
-
-/** Readability: avoid const-casts in code. */
-static inline void free_const(const void *what)
-{
-	free((void *)what);
-}
-
-static inline void *mm_alloc(knot_mm_t *mm, size_t size)
-{
-	if (mm) return mm->alloc(mm->ctx, size);
-	else return malloc(size);
-}
-static inline void mm_free(knot_mm_t *mm, const void *what)
-{
-	if (mm) {
-		if (mm->free)
-			mm->free((void *)what);
-	}
-	else free_const(what);
-}
-
-/** Realloc implementation using memory context. */
-KR_EXPORT
-void *mm_realloc(knot_mm_t *mm, void *what, size_t size, size_t prev_size);
-
-/** Trivial malloc() wrapper. */
-void *mm_malloc(void *ctx, size_t n);
-/** posix_memalign() wrapper. */
-void *mm_malloc_aligned(void *ctx, size_t n);
-
-/** Initialize mm with standard malloc+free. */
-static inline void mm_ctx_init(knot_mm_t *mm)
-{
-	mm->ctx = NULL;
-	mm->alloc = mm_malloc;
-	mm->free = free;
-}
-
-/** Initialize mm with malloc+free with higher alignment (a power of two). */
-static inline void mm_ctx_init_aligned(knot_mm_t *mm, size_t alignment)
-{
-	assert(__builtin_popcount(alignment) == 1);
-	mm->ctx = (uint8_t *)NULL + alignment; /*< roundabout to satisfy linters */
-	/* posix_memalign() doesn't allow alignment < sizeof(void*),
-	 * and there's no point in using it for small values anyway,
-	 * as plain malloc() guarantees at least max_align_t.
-	 * Nitpick: we might use that type when assuming C11. */
-	mm->alloc = alignment > sizeof(void*) ? mm_malloc_aligned : mm_malloc;
-	mm->free = free;
-}
-
-/* @endcond */
+// Use this for alocations with mm.
+// Use mm_alloc for alocations into mempool
 
 /** A strcmp() variant directly usable for qsort() on an array of strings. */
 static inline int strcmp_p(const void *p1, const void *p2)
