@@ -2,9 +2,13 @@
 -- check prerequisites
 local has_http = pcall(require, 'kres_modules.http') and pcall(require, 'http.request')
 if not has_http then
-	pass('skipping http module test because its not installed')
-	done()
+	-- skipping http module test because its not installed
+	os.exit(77)
 else
+	local path = worker.cwd..'/control/'..worker.pid
+	same(true, net.listen(path, nil, {kind = 'control'}),
+		'new control sockets were created so map() can work')
+
 	local request = require('http.request')
 
 	modules.load('http')
