@@ -2,8 +2,8 @@
 -- check prerequisites
 local has_http = pcall(require, 'kres_modules.http') and pcall(require, 'http.request')
 if not has_http then
-	pass('skipping http module test because its not installed')
-	done()
+	-- skipping http module test because its not installed
+	os.exit(77)
 else
 	local request = require('http.request')
 	local openssl_ctx = require('openssl.ssl.context')
