@@ -1,3 +1,4 @@
+-- SPDX-License-Identifier: GPL-3.0-or-later
 -- Module implementing RFC 8145 section 5
 -- Signaling Trust Anchor Knowledge in DNS using Key Tag Query
 local kres = require('kres')
@@ -50,11 +51,13 @@ local function send_ta_query(domain)
 end
 
 -- act on DNSKEY queries which were not answered from cache
-function M.layer.consume(state, req, _)
-	req = kres.request_t(req)
-	local qry = req:current()
-	if qry.stype == kres.type.DNSKEY and not qry.flags.CACHED then
-		send_ta_query(qry:name())
+function M.layer.consume(state, req, pkt)
+	-- First check for standard "cached packets": PKT_SIZE_NOWIRE, for efficiency.
+	if pkt.size ~= -1 and pkt:qtype() == kres.type.DNSKEY then
+		local qry = req:current()
+		if not qry.flags.CACHED then
+			send_ta_query(qry:name())
+		end
 	end
 	return state  -- do not interfere with normal query processing
 end
