@@ -1,9 +1,11 @@
-#!/bin/bash -e
+#!/bin/bash
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 # Example usage:
 # 1. place tarball to be released in git root dir
-# 2. scripts/make-distrofiles.sh -s
+# 2. scripts/make-distrofiles.sh
 # 3. scripts/build-in-obs.sh knot-resolver-latest
+set -o errexit -o nounset -o xtrace
 
 project=home:CZ-NIC:$1
 package=knot-resolver
