@@ -1,10 +1,10 @@
 #!/bin/bash
+# SPDX-License-Identifier: GPL-3.0-or-later
+# Create a source rpm for Fedora/EPEL
 set -o errexit -o nounset -o xtrace
 
-# Create a source rpm for Fedora/EPEL
+cd "$(dirname ${0})/.."
 
-cd "$(git rev-parse --show-toplevel)"
-scripts/make-archive.sh
 scripts/make-distrofiles.sh
 mv knot-resolver_*.orig.tar.xz distro/rpm/
 cd distro/rpm
