@@ -78,8 +78,10 @@ const uint16_t *kr_gc_key_consistent(knot_db_val_t key)
 	} else {
 		/* find the first double zero in the key */
 		for (i = 2; kd[i - 1] || kd[i - 2]; ++i) {
-			if (i >= key.len)
+			if (i >= key.len) {
+				// TODO: assert(!EINVAL) -> kr_assume()
 				return NULL;
+			}
 		}
 	}
 	// the next character can be used for classification
@@ -94,7 +96,10 @@ const uint16_t *kr_gc_key_consistent(knot_db_val_t key)
 		return &NSEC1;
 	case '3':
 		return &NSEC3;
+	case 'S': // the rtt_state entries are considered inconsistent, at least for now
+		return NULL;
 	default:
+		assert(!EINVAL);
 		return NULL;
 	}
 }
@@ -206,6 +211,8 @@ int kr_gc_cache_iter(knot_db_t * knot_db, const  kr_cache_gc_cfg_t *cfg,
 			entry = val2entry(val, *entry_type);
 		}
 		/* TODO: perhaps improve some details around here:
+		 *  - rtt_state entries are considered gc_inconsistent;
+		 *    therefore they'll be the first to get freed (see kr_gc_categorize())
 		 *  - xNAME have .rrtype NS
 		 *  - DNAME hidden on NS name will not be considered here
 		 *  - if zone has NSEC* meta-data but no NS, it will be seen
